#include "PDFinterface.hh"
#include "math/special_functions.hh"
#include <cmath>
#include <vector>
#include "LHAPDF/LHAPDF.h"


using namespace std;



namespace PDFinterface {

  // constructor
  PDFs::PDFs(string PDFset, int member) {
#if LHAPDF_MAJOR_VERSION == 6
    lhpdf = LHAPDF::mkPDF(PDFset, member); 
#else
    PDFset = PDFset + ".LHgrid";
    LHAPDF::setVerbosity(LHAPDF::SILENT);
    LHAPDF::initPDFSet(PDFset, member);
#endif
    for(int i=0; i<13; i++) excludePDF[i] = false;
  }

  PDFs::~PDFs() {
#if LHAPDF_MAJOR_VERSION == 6
    delete lhpdf;
#endif
  }






  double PDFs::xfx(double x, double Q, int parton) {
    if(excludePDF[6+parton]) return 0;
#if LHAPDF_MAJOR_VERSION == 6
    //if(parton == 0) parton = 21;
    return lhpdf->xfxQ(parton, x, Q);
#else
    return LHAPDF::xfx(x,Q,parton);
#endif
  }



  void PDFs::xfx(double x, double Q, double *pdfs) {
#if LHAPDF_MAJOR_VERSION == 6
    vector<double> rtn;
    lhpdf->xfxQ(x, Q, rtn);
    for(int i=0; i<=12; i++) {
      pdfs[i] = ( excludePDF[i] ? 0 : rtn[i] );
    }
#else
    LHAPDF::xfx(x,Q,pdfs);
    for(int i=0; i<=12; i++) {
      if(excludePDF[i]) pdfs[i] = 0;
    }
#endif
    return;
  }




  double PDFs::ConvertNucleonPDF(unsigned int parton, double xd, double xu, nucleon n) {
    if(parton > 2) abort();
    if(n==Proton) {
      if(parton==1) return xd;
      if(parton==2) return xu;
    }
    if(n==Neutron) {
      if(parton==1) return xu; // swap ud and down
      if(parton==2) return xd;
    }
    if(n==Deuteron) {
      return (xu+xd)/2.;
    }
    if(n==Iron) {
      double nP=26, nPN=55.845;
      if(parton==1) return (nP*xu + (nPN-nP)*xd)/nPN;
      if(parton==2) return (nP*xd + (nPN-nP)*xu)/nPN;
    }
    abort();
    return 0;
  }


  double PDFs::xfx(double x, double Q, int parton, nucleon n, bool antiparticle) {
    if(antiparticle) parton *= -1;
    if(n>0 && (fabs(parton) == 1 || fabs(parton) == 2)) {
      double xd = xfx(x,Q,sign(parton)*1);
      double xu = xfx(x,Q,sign(parton)*2);
      return ConvertNucleonPDF(fabs(parton), xd, xu, n);
    }
    else return xfx(x,Q,parton);
  }

  void PDFs::xfx(double x, double Q, double *pdf, nucleon n, bool antiparticle) {
    xfx(x,Q,pdf);
    if(antiparticle) {
      double tmp;
      for(int i=1; i<=6; i++) {
	tmp = pdf[6+i];
	pdf[6+i] = pdf[6-i];
	pdf[6-i] = tmp;
      }
    }
    if(n>0) {
      double xd  = pdf[6+1];
      double xu  = pdf[6+2];
      double xdb = pdf[6-1];
      double xub = pdf[6-2];
      pdf[6+1] = ConvertNucleonPDF(1, xd , xu , n);
      pdf[6+2] = ConvertNucleonPDF(2, xd , xu , n);
      pdf[6-1] = ConvertNucleonPDF(1, xdb, xub, n);
      pdf[6-2] = ConvertNucleonPDF(2, xdb, xub, n);
    }
    return;
  }

  // get Quark masses from LHAPDF
  double PDFs::getQuarkMass(int flavour) {
    flavour = fabs(flavour);
#if LHAPDF_MAJOR_VERSION == 6
    string flav[] = {"","MDown","MUp","MStrange","MCharm","MBottom","MTop"};
    //cout << flavour << "   " << flav[flavour] << endl;
    return lhpdf->info().get_entry_as<double>(flav[flavour]);
#else
    return LHAPDF::getQMass(flavour);
#endif
  }

  // get Quark thresholds from LHAPDF (currently they are just the masses..)
  double PDFs::getQuarkThreshold(int flavour) {
    flavour = fabs(flavour);
#if LHAPDF_MAJOR_VERSION == 6
    string flav[] = {"","MDown","MUp","MStrange","MCharm","MBottom","MTop"};
    return lhpdf->info().get_entry_as<double>(flav[flavour]);
#else
    return LHAPDF::getQMass(flavour);
#endif
  }

  // alphas from LHAPDF
  double PDFs::alphasQ(double mu) {
#if LHAPDF_MAJOR_VERSION == 6
    return lhpdf->alphasQ(mu);
#else
    return LHAPDF::alphasPDF(mu);
#endif
  }



  void PDFs::TurnOnOffPDFs(bool *activePDF) {
    for(int i=0; i<13; i++) {
      excludePDF[i] = !activePDF[i];
    }
  }


};


